// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_ABSTRACT_BASE_H
#define OB_ABSTRACT_BASE_H

namespace CORBA
{

class AbstractBase;
typedef AbstractBase* AbstractBase_ptr;

} // End of namespace CORBA

void OBRelease(CORBA::AbstractBase_ptr);
void OBDuplicate(CORBA::AbstractBase_ptr);

namespace CORBA
{

typedef OB::ObjVar< AbstractBase > AbstractBase_var;


class AbstractBase
{
    //
    // Not implemented
    //
    void operator=(const AbstractBase&);

protected:
    AbstractBase();
    AbstractBase(const AbstractBase&);
    
    virtual ~AbstractBase() = 0;

public:
    static AbstractBase_ptr _duplicate(AbstractBase_ptr);
    static AbstractBase_ptr _narrow(AbstractBase_ptr);
    static AbstractBase_ptr _nil();

    //
    // These are used to distinguish whether an abstract interface reference
    // refers to an object reference or a valuetype.  The appropriate
    // function must be overridden in concrete derived classes.
    //
    virtual Object_ptr _OB_toObject();
    virtual ValueBase* _OB_toValue();

    //
    // Instances of types derived from an abstract interface behave
    // polymorphically when inserting into an Any
    //
    virtual void _OB_insertAny(Any&) = 0;
};

Boolean is_nil(AbstractBase_ptr);
void release(AbstractBase_ptr);

} // End of namespace CORBA

#endif
